{-|
Module      : Gargantext.Core.Config.Ini.Mail
Description :
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}

module Gargantext.Core.Config.Ini.Mail (
  -- * Types
    GargMail(..)
  , LoginType(..)
  , MailConfig(..)

  -- * Utility function
  , readConfig
  )
  where

import Data.Maybe

import Gargantext.Core.Config.Ini.Ini (readIniFile', val)
import Gargantext.Core.Config.Mail (LoginType(..), MailConfig(..), SendEmailType(LogEmailToConsole))
import Gargantext.Prelude
import Prelude (read)


type Email = Text
type Name  = Text

readConfig :: FilePath -> IO MailConfig
readConfig fp = do
  ini <- readIniFile' fp
  let val' = val ini "mail"

  pure $ MailConfig { _mc_mail_host       =        cs $ val' "MAIL_HOST"
                    , _mc_mail_port       = read $ cs $ val' "MAIL_PORT"
                    , _mc_mail_user       =        cs $ val' "MAIL_USER"
                    , _mc_mail_from       =        cs $ val' "MAIL_FROM"
                    , _mc_mail_password   =        cs $ val' "MAIL_PASSWORD"
                    , _mc_mail_login_type = read $ cs $ val' "MAIL_LOGIN_TYPE"
                    , _mc_send_login_emails = LogEmailToConsole
                    }


data GargMail = GargMail { gm_to      :: Email
                         , gm_name    :: Maybe Name
                         , gm_subject :: Text
                         , gm_body    :: Text
                         }
