{-|
Module      : Gargantext.Core.Methods.Similarities.Accelerate.Conditional
Description : 
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

This module aims at implementig distances of terms context by context is
the same referential of corpus.

Implementation use Accelerate library which enables GPU and CPU computation
See Gargantext.Core.Methods.Graph.Accelerate)

-}

{-# LANGUAGE TypeFamilies        #-}
{-# LANGUAGE ScopedTypeVariables #-}

module Gargantext.Core.Methods.Similarities.Accelerate.Conditional
  where

-- import qualified Data.Foldable as P (foldl1)
-- import Debug.Trace (trace)
import Data.Array.Accelerate
import Data.Array.Accelerate.Interpreter (run)
import Gargantext.Core.Methods.Matrix.Accelerate.Utils
import Gargantext.Core.Methods.Similarities.Accelerate.SpeGen


-- * Metrics of proximity
-----------------------------------------------------------------------
-- ** Conditional distance

-- *** Conditional distance (basic)

-- | Conditional distance (basic version)
--
-- 2 main metrics are actually implemented in order to compute the
-- proximity of two terms: conditional and distributional
--
-- Conditional metric is an absolute metric which reflects
-- interactions of 2 terms in the corpus.

-- Filtered with MiniMax.


measureConditional :: Matrix Int -> Matrix Double
measureConditional m =  run $ x $ map fromIntegral $ use m
  where
    x :: Acc (Matrix Double) -> Acc (Matrix Double)
    x mat = maxOnly $ diagNull r $ divByDiag r mat

    r :: Dim
    r = dim m

    -- Maybe we should use backpermute to accelerate it (no need to access to cells then
    maxOnly :: Acc (SymetricMatrix Double) -> Acc (Matrix Double)
    maxOnly m' = generate (shape m')
                          ((\coord 
                             -> let (Z :. (i :: Exp Int) :. (j :: Exp Int)) = unlift coord
                                    ij = m' ! (lift $ (Z :. i :. j))
                                    ji = m' ! (lift $ (Z :. j :. i))
                                    in 
                                      ifThenElse (ij > ji) ij (constant 0)
                            )
                            )

measureConditional' :: Matrix Int -> Matrix Double
measureConditional' m =  run $ x $ map fromIntegral $ use m
  where
    x :: Acc (Matrix Double) -> Acc (Matrix Double)
    x mat = matMiniMax $ matProba r mat

    r :: Dim
    r = dim m
